function data = run_Murd62(param)
% data = run_Murd62(param)
%
% Simulate the Murdock (1962) experiment using the CMR model.  
%
% param - a parameters structure specifying the configuration of
% the model.
%
% Example:
%
% param = param_cmr_m62;
% net_data = run_Murd62(param);
%
% param.LL = [20 30 40];
% datapath = '';
%
%

rand('state',sum(1000*clock));
num_trials = param.num_trials;

% load the semantic structures
for i=1:param.subregions
  param.sem_mat{i} = [];
  if param.has_semantic_structure(i)
    sfile = load(param.sem_path{i});
    param.sem_mat{i} = sfile.sem_mat;
  end  
end

for i = 1:length(param.LL)
  
  fprintf('running LL %d:\n',param.LL(i))
  
  param.pres_indices = cell(1);
  param.pres_indices{1} = zeros(param.subregions, param.LL(i));

  for j = 1:num_trials
  
    % set the item pres_indices
    param.pres_indices{1}(1,:) = randsample(size(param.sem_mat{1},1),param.LL(i));

    % determine the appropriate number of patterns
    [param.n_patterns param.first_distraction_index] ...
        = calculate_session_patterns(param);

    % create the environment
    env = create_orthogonal_patterns(param.n_patterns, ...
                                     param.pres_indices, ...
                                     param.not_presented_indices);
    if param.orthogonal_patterns
      param.n_dimensions = param.n_patterns;
    else
      param.n_dimensions = size(env.patterns{1},2);
    end
    env.init_index = 0;

    % run trials one at a time
    temp_trial = simulate_fr(param, env);
    temp_trial.subject = 1;
    temp_trial.pres_itemnos = param.pres_indices{1}(1,:);
    
    trial(j) = temp_trial;
    
    if mod(j,50)==1
      fprintf('%d ',j);
    end
    
  end
  
  fprintf('\n');
    
  % take this set of trials and place it on an output structure
  f = fieldnames(trial(1));
  for j = 1:length(f)
    data.LL(i).(f{j}) = [];
  end
  
  % find max number of columns
  maxcols = 0;
  for j = 1:length(trial)
    ncols = size(trial(j).recalls,2);
    maxcols = max(ncols, maxcols);
  end

  % stitch them together, with appropriate padding
  for j = 1:length(trial)
    for k = 1:length(f)
      this_field = trial(j).(f{k});
      ncols = size(this_field,2);
      if ncols < maxcols
        this_field(:,ncols+1:maxcols) = 0;
      end
      data.LL(i).(f{k}) = [data.LL(i).(f{k}); this_field];
    end
  end

  % don't pad subjects
  data.LL(i).subject = data.LL(i).subject(:,1);


end % end list lengths
  
 
  
